/*
 * EllipsisUnitScaler.java
 *
 * Created on 30 May 2005, 15:04
 *
 * Copyright (C) 2005 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Authors:
 *      James Clark <jclark@geosci.usyd.edu.au>
 *
 */

import java.util.HashMap;
import java.util.Map;

/**
 *  This abstract class is used to define how the scaling code can be interacted with.
 *  The idea is, if we later want to use some alternative scaling methods,
 *  we can just write a drop-in replacement class implementing this interface.
 *
 * @author jclark
 */
public abstract class EllipsisUnitScaler {
    
    /** Mandatory for implementations to provide this.
     *  Name should include some sort of version number so we know when things become incompatible
     */
    public String getName()
    {
        return "Abstract EllipsisUnitScaler 1.0";
    }
    
    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter.
     *  This will most likely just be used to automagically store scaling parameter settings in a file.
     */
    public String[] getScalingParameters()
    {
        String[] params = {  };
        return params;
    }

    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter, same order as above!!
     *  This should be a short text description of the parameter to be used as a label in the GUI
     */
    public String[] getScalingParameterDescriptions()
    {
        String[] descriptions = {  };
        return descriptions;
    }

    /** The constructor for this abstract class MUST be called via a call to
     *      super();
     *  in your implementation.
     */
    public EllipsisUnitScaler()
    {
        independentScalingParameters = new HashMap();
    }


    
    /** Setup function
     *  Call this with some sort of identifier for each independent scaling parameter.
     *  This will vary per implementation, but as an example,
     *  setScalingParameter("Length", 150000);
     *  setScalingParameter("Temperature", 1330 + 273);
     *  In reality these independent variables for scaling will be set from the GUI.
     *  This should NOT be confused with the sLength(), sGravity() methods! this is for
     *  setting the implementation-specific independent variables only!
     *
     *  @param parameter the name of the parameter to set
     *  @param value the numeric value for that parameter to be set to.
     */
    public void setScalingParameter(String parameter, double value)
    {
        independentScalingParameters.put(parameter, new Double(value));
    }
    
    /** Mostly-Internal function
     *  Used by sLength(), sGravity() etc to retrieve the user-set independent scaling parameters.
     *  Call this with your identifier for the desired parameter.
     *
     *  The only methods available to get the scaling factors for use in si2elu() and elu2si()
     *  should be the sSomethingSomething() methods!
     *
     *  The only purpose for this method being public is to retrieve the scaling parameters,
     *  eg for writing to file. actual scaling should still be done with all the sSomethingSomething() methods.
     *
     *  Yes bad things will happen if you give it a phony or misspelled name!
     *
     *  @param parameter the name of the parameter to set
     */
    public double getScalingParameter(String parameter)
    {
        return ((Double)independentScalingParameters.get(parameter)).doubleValue();
    }

    
    /** Main conversion function
     *  The one we'll be using most- go from the real-world(in the gui) to ellipsis units (i'm calling it "elu" ;) JC
     *
     *  @param si the SI value to be converted to Ellipsis Units
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    public double si2elu(double si, double scalingFactor)
    {
        return si / scalingFactor;
    }
    
    /** Main conversion function
     *  The one we likely won't care about- go from ellipsis units(in the file) to real-world(in the gui)
     *
     *  @param elu the Ellipsis Units value to be converted to SI
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    public double elu2si(double elu, double scalingFactor)
    {
        return elu * scalingFactor;
    }
    
    
    /** Scaling Factors
     *  One function needs to be written for each scaling factor, dependent or independent.
     *  This will return the appropriate factor as a double.
     *  You use them in a call to either si2elu() or elu2si().
     */
    public abstract double sLength();
    public abstract double sGravity();
    public abstract double sThermalDiffusity();
    public abstract double sTemperature();
    public abstract double sViscosity();
    
    public abstract double sTime();
    public abstract double sDensity();
    public abstract double sStress();
    public abstract double sVelocity();
    public abstract double sEnergy();
    public abstract double sThermalExpansionCoefficient();
    public abstract double sStrainRate();
    public abstract double sSpecificHeat();
    public abstract double sActivationEnergy();
    public abstract double sHeatGeneration();
    public abstract double sHeatFlux();
    public abstract double sThermalConductivity();
    public abstract double sThermalEnergy();
    public abstract double sInverseTemperature();

    /** Not for scaling (dimensionless)
     */
    public double sNull()                                   /** NULL SCALER (unitless or not for scaling) */
    {
        return 1.0;
    }
    
    
    
    
    /** Access this using setScalingParameter() and getScalingParameter()
     */
    protected Map independentScalingParameters;
    
}
