/*
 * EluScaling.java
 *
 * Created on 30 May 2005, 16:03
 *
 * Copyright (C) 2005 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Authors:
 *      James Clark <jclark@geosci.usyd.edu.au>
 *
 */


/**
 *  This class is in charge of the scaling logic.
 *  it implements the interface defined in EllipsisUnitScaler by inheriting from it.
 *  just to ensure everything can be consistent and just slightly future proof enough.
 *  The idea is, if we later want to use some alternative scaling methods,
 *  we can just write a drop-in replacement class implementing this interface.
 *
 *  See EllipsisUnitScaler for the full interface; users of this class will most likely
 *  be taking the following initialisation steps:
 *
 *      es = new EluScaling();
 *      es.setScalingParameter("Length", 150e3);
 *      es.setScalingParameter("Gravity", 1);
 *      es.setScalingParameter("ThermalDiffusity", 1e-6);
 *      es.setScalingParameter("Temperature", 1330 + 273);
 *      es.setScalingParameter("Viscosity", 1e21);
 *
 *  then repeated calls of the form:
 *      scaled_depth = es.si2elu(DepthBox.doubleValue(), es.sLength());
 *  or
 *      unscaled_depth = es.elu2si(scaled_depth, es.sLength());
 *
 *  Although what actually will happen, is we wrap the si2elu() call in ellipsisgui's ScalingUI class,
 *  as a toFile() method, and the elu2si() call becomes a fromFile() method. the elus.sLength() etc
 *  will be unchanged. Why do this? Because ScalingUI is smart enough to know when to scale, and
 *  when to leave alone. EllipsisUnitScaler is more pure and does not concern itself with worldly things
 *  such as radio buttons controling scaling.
 *  So, we'd get something like:
 *      settings+="visc_min="+scaling.viscosity2EllipsisUnits(minimumViscosityTextField.getText())+"\n";
 *  becoming:
 *      settings+="visc_min="+scaling.toFile(minimumViscosityTextField.getText(), elus.sViscosity())+"\n";
 *
 * @author jclark
 */
public class EluScaling extends EllipsisUnitScaler {
    
    /** Mandatory for implementations to provide this.
     *  Name should include some sort of version number so we know when things become incompatible
     */
    public String getName()
    {
        return "USIMS EluScaling 1.0";
    }
    
    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter.
     *  This will most likely just be used to automagically store scaling parameter settings in a file.
     */
    public String[] getScalingParameters()
    {
        String[] params = { "Length",
                            "Gravity",
                            "ThermalDiffusity",
                            "Temperature",
                            "Viscosity"
                          };
        return params;
    }
        
    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter, same order as above!!
     *  This should be a short text description of the parameter to be used as a label in the GUI
     */
    public String[] getScalingParameterDescriptions()
    {
        String[] descriptions = {   "Length             (metres)",
                                    "Gravity            (metres/seconds\u00B2)",
                                    "Thermal Diffusity  (meters\u00B2/seconds)",
                                    "Temperature        (kelvin)",
                                    "Viscosity          (pascal/seconds)"
                                };
        return descriptions;
    }

    /** Creates a new instance of EluScaling
     *  note that we MUST call super(); in our implementation.
     */
    public EluScaling()
    {
        super();
        
        // So we don't forget what they are called,
        // Let's set some defaults for our independent scaling parameters:
        setScalingParameter("Length", 150e3);
        setScalingParameter("Gravity", 1);
        setScalingParameter("ThermalDiffusity", 1e-6);
        setScalingParameter("Temperature", 1330 + 273);
        setScalingParameter("Viscosity", 1e21);
    }
    
    
    // we don't need to change the definitions of these, but they're here...

    /** Setup function
     *  Call this with some sort of identifier for each independent scaling parameter.
     *  This will vary per implementation, but as an example,
     *  setScalingParameter("Length", 150000);
     *  setScalingParameter("Temperature", 1330 + 273);
     *  In reality these independent variables for scaling will be set from the GUI.
     *  This should NOT be confused with the sLength(), sGravity() methods! this is for
     *  setting the implementation-specific independent variables only!
     *
     *  @param parameter the name of the parameter to set
     *  @param value the numeric value for that parameter to be set to.
     */
    //  public void setScalingParameter(String parameter, double value);

    
    /** Main conversion function
     *  The one we'll be using most- go from the real-world(in the gui) to ellipsis units (i'm calling it "elu" ;)
     *
     *  @param si the SI value to be converted to Ellipsis Units
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    //  public double si2elu(double si, double scalingFactor);

    
    /** Main conversion function
     *  The one we likely won't care about- go from ellipsis units(in the file) to real-world(in the gui)
     *
     *  @param elu the Ellipsis Units value to be converted to SI
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    //  public double elu2si(double elu, double scalingFactor);
    

    
    
    
    /** Scaling Factors
     *  One function needs to be written for each scaling factor, dependent or independent.
     *  This will return the appropriate factor as a double.
     *  You use them in a call to either si2elu() or elu2si().
     */
    
    /** These are our independent scaling parameters.
     *  We just get the value that the user set via setScalingParameter().
     */
    
    public double sLength()                                 /** LENGTH (meters) */
    {
        return getScalingParameter("Length");
    }
    public double sGravity()                                /** GRAVITY (meters/seconds^2) */
    {
        return getScalingParameter("Gravity");
    }
    public double sThermalDiffusity()                       /** THERMAL DIFFUSITY (meters^2/seconds) */
    {
        return getScalingParameter("ThermalDiffusity");
    }
    public double sTemperature()                            /** TEMPERATURE (kelvin) */
    {
        return getScalingParameter("Temperature");
    }
    public double sViscosity()                              /** VISCOSITY (pascal/seconds == kilogram/meter.second^2) */
    {
        return getScalingParameter("Viscosity");
    }

    
    /** These are our dependent scaling factors.
     *  We compute their value based on the independent factors eg sLength(),
     *  and also based on each other.
     *  For this reason it is VERY important to not accidentally create infinite recursion!
     */
    
    public double sTime()                                   /** TIME (seconds) */
    {
        return (sLength()*sLength()) / sThermalDiffusity();
    }
    public double sDensity()                                /** DENSITY (kilograms/metre^3) */
    {
        return (sViscosity() / (sGravity()*sLength()*sTime()));
    }
    public double sStress()                                 /** STRESS (pascals = kilogram/metre.second^2) */
    {
        return (sDensity()*sGravity()*sLength());
    }
    public double sVelocity()                               /** VELOCITY (metres/second) */
    {
        return (sLength() / sTime());
    }
    public double sEnergy()                                 /** ENERGY (metres^2.kilogram/seconds^2) */
    {
        return ((sDensity()*sLength()*sLength()*sLength()) * sGravity()*sLength());
    }
    public double sThermalExpansionCoefficient()            /** THERMAL EXPANSION COEFFICIENT (1/kelvin) */
    {
        return (1/sTemperature());
    }
    public double sStrainRate()                             /** STRAIN RATE (seconds) */
    {
        return (sTime());
    }
    public double sSpecificHeat()                           /** SPECIFIC HEAT (metres^2/kelvin.seconds^2) */
    {
        return (sStress() / (sDensity()*sTemperature()));
    }
    public double sActivationEnergy()                       /** ACTIVATION ENERGY (metres^2.kilogram/seconds^2) */
    {
        return (sTemperature());
    }
    public double sHeatGeneration()                         /** HEAT GENERATION (watts/kilogram = metres^2/seconds^3) */
    {
        return (sStress() / (sDensity()*sTime()));
    }
    public double sHeatFlux()                               /** HEAT FLUX () */
    {
        return ((sSpecificHeat()*sDensity()*sTemperature()*sLength()) / sTime());
    }
    public double sThermalConductivity()                    /** THERMAL CONDUCTIVITY (metres.kilogram/kelvin.seconds^3) */
    {
        return (sThermalDiffusity()*sDensity()*sSpecificHeat());
    }
    public double sThermalEnergy()                          /** THERMAL ENERGY () */
    {
        return (sSpecificHeat()*sDensity()*sLength()*sLength()*sLength()*sTemperature());
    }
    public double sInverseTemperature()                     /** INVERSE TEMPERATURE (1/kelvin) used for MaterialRheologyWindow/rheology phase viscT1 */
    {
        return (1/sTemperature());
    }

    /** Not for scaling (dimensionless)
     */
    public double sNull()                                   /** NULL SCALER (unitless or not for scaling) */
    {
        return 1.0;
    }
    
}

