/*
 * NoScaling.java
 *
 * Created on 23 June 2005, 16:39
 *
 * Copyright (C) 2005 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Authors:
 *      James Clark <jclark@geosci.usyd.edu.au>
 *
 */


/**
 *  This class is in charge of the scaling logic.
 *  it implements the interface defined in EllipsisUnitScaler by inheriting from it.
 *  just to ensure everything can be consistent and just slightly future proof enough.
 *  The idea is, if we later want to use some alternative scaling methods,
 *  we can just write a drop-in replacement class implementing this interface.
 *
 *  This special "No Scaling" class implements a scaling model where everything is left unscaled.
 *  It serves to simplify the method of disabling ellipsisgui's scaling,
 *  and also to illustrate an alternative use of the EllipsisUnitScaler interface.
 *
 * @author jclark
 */
public class NoScaling extends EllipsisUnitScaler {
    
    /** Mandatory for implementations to provide this.
     *  Name should include some sort of version number so we know when things become incompatible
     */
    public String getName()
    {
        return "No Scaling"; // this name is checked for in the GUI, to ensure the status label is meaningful.
    }
    
    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter.
     *  This will most likely just be used to automagically store scaling parameter settings in a file.
     */
    public String[] getScalingParameters()
    {
        String[] params = {
                          };
        return params;
    }
        
    /** Another important static information providing function.
     *  Should return an array of String, one for each settable parameter, same order as above!!
     *  This should be a short text description of the parameter to be used as a label in the GUI
     */
    public String[] getScalingParameterDescriptions()
    {
        String[] descriptions = {
                                };
        return descriptions;
    }

    /** Creates a new instance of NoScaling
     *  note that we MUST call super(); in our implementation.
     */
    public NoScaling()
    {
        super();
        
        // So we don't forget what they are called,
        // Let's set some defaults for our independent scaling parameters:
        // (except there aren't any for No Scaling)
    }
    
    
    // we don't need to change the definitions of these, but they're here...

    /** Setup function
     *  Call this with some sort of identifier for each independent scaling parameter.
     *  This will vary per implementation, but as an example,
     *  setScalingParameter("Length", 150000);
     *  setScalingParameter("Temperature", 1330 + 273);
     *  In reality these independent variables for scaling will be set from the GUI.
     *  This should NOT be confused with the sLength(), sGravity() methods! this is for
     *  setting the implementation-specific independent variables only!
     *
     *  @param parameter the name of the parameter to set
     *  @param value the numeric value for that parameter to be set to.
     */
    //  public void setScalingParameter(String parameter, double value);

    
    /** Main conversion function
     *  The one we'll be using most- go from the real-world(in the gui) to ellipsis units (i'm calling it "elu" ;)
     *
     *  NoScaling overrides this method to perform it's own unique system of scaling.
     *  (i.e., do nothing!)
     *
     *  @param si the SI value to be converted to Ellipsis Units
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    public double si2elu(double si, double scalingFactor)
    {
        return si;
    }

    
    /** Main conversion function
     *  The one we likely won't care about- go from ellipsis units(in the file) to real-world(in the gui)
     *
     *  NoScaling overrides this method to perform it's own unique system of scaling.
     *  (i.e., do nothing!)
     *
     *  @param elu the Ellipsis Units value to be converted to SI
     *  @param scalingFactor the scaling factor appropriate for the units of this conversion. Obtain via the other functions in this interface.
     */
    public double elu2si(double elu, double scalingFactor)
    {
        return elu;
    }
    

    
    
    
    /** Scaling Factors
     *  One function needs to be written for each scaling factor, dependent or independent.
     *  This will return the appropriate factor as a double.
     *  You use them in a call to either si2elu() or elu2si().
     */
    
    /** In NoScaling, we do things differently from the scaling-factor that EluScaling uses.
     *  Rather than return the actual divisor, we can return a simple integer flag.
     *  NoScaling won't need to look at these flags of course, but it could be helpful for debugging.
     *
     *  and yes, there are no independent scaling parameters
     */
    
    public double sLength()
    {
        return 1;
    }
    public double sGravity()
    {
        return 2;
    }
    public double sThermalDiffusity()
    {
        return 3;
    }
    public double sTemperature()
    {
        return 4;
    }
    public double sViscosity()
    {
        return 5;
    }
    public double sTime()
    {
        return 6;
    }
    public double sDensity()
    {
        return 7;
    }
    public double sStress()
    {
        return 8;
    }
    public double sVelocity()
    {
        return 9;
    }
    public double sEnergy()
    {
        return 10;
    }
    public double sThermalExpansionCoefficient()
    {
        return 11;
    }
    public double sStrainRate()
    {
        return 12;
    }
    public double sSpecificHeat()
    {
        return 13;
    }
    public double sActivationEnergy()
    {
        return 14;
    }
    public double sHeatGeneration()
    {
        return 15;
    }
    public double sHeatFlux()
    {
        return 16;
    }
    public double sThermalConductivity()
    {
        return 17;
    }
    public double sThermalEnergy()
    {
        return 18;
    }
    public double sInverseTemperature()
    {
        return 19;
    }

    /** Not for scaling (dimensionless)
     */
    public double sNull()                                   /** NULL SCALER (unitless or not for scaling) */
    {
        return -1;
    }
    
}

