/*
 * Bean.java
 *
 * Created on December 2, 2003, 2:34 PM
 */

/* This class extends JTextField, to provide a Java Bean which can be used for storing numeric values.
 * The text must always be set to a valid numeric value (with setText() ), and its value can be retrieved (in 
 * text format) with getText(). Any attempt to set it to an invalid value (such as setting it to some string 
 * which is not a valid number) will result in a warning message being given in the form of a JOptionPane, and the
 * value remaining uncahnged.
 *
 * Properties can be set to restrict the range of allowable numbers. These are
 *
 * negative numbers can be dissallowed
 * fractional numbers can be dissallowed
 * zero can be dissallowed
 *
 * a minmum value can be specified
 * a maximum value can be specified
 *
 * Integers and doubles are treated separately to avoid any rounding or casting problems.
 *
 * The parent window member exists so that the warning messages can be modal, but no way to initialise this
 * value yet exists.
 *
 * Copyright (C) 2003 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Author:
 *   Mike Dowman <mdowman@geosci.usyd.edu.au>
 *
*/

import javax.swing.*;
/**
 *
 * @author  mdowman
 */
public class NumberBoxBean extends JTextField implements java.io.Serializable {
    
    private java.awt.Component parentWindow; // This is needed so that modal JOptionPanes can be opened warning about bad settings.
    
    private boolean zeroIsAllowed=true;
    private boolean negativeNumbersAreAllowed=true;
    private boolean fractionalNumbersAreAllowed=true;
    
    private boolean minimumValueIsSpecified=false;
    private int minimumIntegerValue;
    private double minimumDoubleValue;
    
    private boolean maximumValueIsSpecified=false;
    private int maximumIntegerValue;
    private double maximumDoubleValue;
    
    private int integerValue; // One of these will record the actual value of the number represented in the box. Separate fields are used for integers and doubles to prevent any rounding problems.
    private double doubleValue;
    
    private void parseCheckAndSetDouble() {
        double newValue;
        try {
            newValue=Double.parseDouble(getText());
        } catch (NumberFormatException ex) { // If the entered text doesn't conform to a valid number we want to warn about this, and restore the text to refect the current value.
            super.setText(""+doubleValue);
            MessageBoxes.warnAboutBadDouble(parentWindow);
            return;
        }
        
        if (!zeroIsAllowed) {
            if (newValue==0) { // If we're not allowed zero, but zero has been entered, warn about this, put the old value back in the text box, and then return.
                super.setText(Useful.double2String(doubleValue));
                MessageBoxes.warnAboutZero(parentWindow);
                return;
            }
        }
        
        if (!negativeNumbersAreAllowed) { // If negative values are not allowed, but one has been entered, warn about this, reset the text, and return.
            if (newValue<0) { 
                super.setText(Useful.double2String(doubleValue));
                MessageBoxes.warnAboutNegative(parentWindow);
                return;
            }
        }
        
        if (minimumValueIsSpecified) {
            if (newValue<minimumDoubleValue) {
                super.setText(Useful.double2String(doubleValue));
                MessageBoxes.warnAboutDoubleBelowMinimum(minimumDoubleValue, parentWindow);
                return;
            }
        }
    
        if (maximumValueIsSpecified) {
            if (newValue>maximumDoubleValue) {
                super.setText(Useful.double2String(doubleValue));
                MessageBoxes.warnAboutDoubleAboveMaximum(maximumDoubleValue, parentWindow);
                return;
            }
        }
        
        doubleValue=newValue; // If the new double value conforms to all the restrictions, set the double value to reflect the new value and leave the text unchanged.
    }
    
    private void parseCheckAndSetInteger() {
        int newValue;
        try {
            newValue=Integer.parseInt(getText());
        } catch (NumberFormatException ex) { // If the entered text doesn't conform to a valid number we want to warn about this, and restore the text to refect the current value.
            super.setText(""+integerValue);
            MessageBoxes.warnAboutBadInteger(parentWindow);
            return;
        }
        
        if (!zeroIsAllowed) {
            if (newValue==0) { // If we're not allowed zero, but zero has been entered, warn about this, put the old value back in the text box, and then return.
                super.setText(""+integerValue);
                MessageBoxes.warnAboutZero(parentWindow);
                return;
            }
        }
        
        if (!negativeNumbersAreAllowed) { // If negative values are not allowed, but one has been entered, warn about this, reset the text, and return.
            if (newValue<0) { 
                super.setText(""+integerValue);
                MessageBoxes.warnAboutNegative(parentWindow);
                return;
            }
        }
        
        if (minimumValueIsSpecified) {
            if (newValue<minimumIntegerValue) {
                super.setText(""+integerValue);
                MessageBoxes.warnAboutDoubleBelowMinimum( (double)minimumIntegerValue, parentWindow);
                return;
            }
        }
    
        if (maximumValueIsSpecified) {
            if (newValue>maximumIntegerValue) {
                super.setText(""+integerValue);
                MessageBoxes.warnAboutDoubleAboveMaximum( (double)maximumIntegerValue, parentWindow);
                return;
            }
        }
        
        integerValue=newValue; // If the new integer value conforms to all the restrictions, set the integer value to reflect the new value and leave the text unchanged.
    }
    
    /** Creates new Box */
    public NumberBoxBean() {
        super.setText("0");
        
        addFocusListener(new java.awt.event.FocusAdapter() { // We want to be warned when the focus is lost.
            public void focusLost(java.awt.event.FocusEvent evt) {
                numberBoxBeanFocusLost(evt);
            }
        });
        
    }
    
    private void numberBoxBeanFocusLost(java.awt.event.FocusEvent event) {
        parseCheckAndSetNewValue();
    }
    
    private void parseCheckAndSetNewValue() {
        if (fractionalNumbersAreAllowed) { // Exactly what we do depends on whether the box holds integers or doubles.
            parseCheckAndSetDouble();
        } else { // If the box holds just integers
            parseCheckAndSetInteger();
        }
    }
    
    public void setMaximumIntegerValue(int newMaximumValue) {
        maximumIntegerValue=newMaximumValue;
    }
    
    public int getMaximumIntegerValue() {
        return maximumIntegerValue;
    }
    
    public void setMinimumIntegerValue(int newMinimumValue) {
        minimumIntegerValue=newMinimumValue;
    }
    
    public int getMinimumIntegerValue() {
        return minimumIntegerValue;
    }
    
    public void setMaximumDoubleValue(double newMaximumValue) {
        maximumDoubleValue=newMaximumValue;
    }
    
    public double getMaximumDoubleValue() {
        return maximumDoubleValue;
    }
    
    public void setMinimumDoubleValue(double newMinimumValue) {
        minimumDoubleValue=newMinimumValue;
    }
    
    public double getMinimumDoubleValue() {
        return minimumDoubleValue;
    }
    
    public void setZeroIsAllowed(boolean newZeroIsAllowed) {
        zeroIsAllowed=newZeroIsAllowed;
    }
    
    public boolean getZeroIsAllowed() {
        return zeroIsAllowed;
    }
    
    public void setNegativeNumbersAreAllowed(boolean newNegativeNumbersAreAllowed) {
        negativeNumbersAreAllowed=newNegativeNumbersAreAllowed;
    }
    
    public boolean getNegativeNumbersAreAllowed() {
        return negativeNumbersAreAllowed;
    }
    
    public void setFractionalNumbersAreAllowed(boolean newFractionalNumbersAreAllowed) {
        fractionalNumbersAreAllowed=newFractionalNumbersAreAllowed;
        parseCheckAndSetNewValue(); // This ensures that the appropriate cache, either the double value or the intger value field is updated.
    }
    
    public boolean getFractionalNumbersAreAllowed() {
        return fractionalNumbersAreAllowed;
    }
    
    public void setMinimumValueIsSpecified(boolean newMinimumValueIsSpecified) {
        minimumValueIsSpecified=newMinimumValueIsSpecified;
    }
    
    public boolean getMinimumValueIsSpecified() {
        return minimumValueIsSpecified;
    }
    
    public void setMaximumValueIsSpecified(boolean newMaximumValueIsSpecified) {
        maximumValueIsSpecified=newMaximumValueIsSpecified;
    }
    
    public boolean getMaximumValueIsSpecified() {
        return maximumValueIsSpecified;
    }
    
    public void setText(String newText) { // If the text is changed by the program, not by the user, we still want to parse it and cash the value, possibly issuing a warning in the process.
        super.setText(newText);
        parseCheckAndSetNewValue();
    }
}
